function p = SRVF_to_curve(q,varargin)
%SRVF_TO_CURVE restores the curve from its Square-Root Velocity Function.
%
%   CURVE = SRVF_TO_CURVE(Q) restores the curve from its Square-Root Velocity 
%   Function (SRVF) representation (see Srivastava et al, PAMI 2011) 
%   using the trapezoidal rule for integration. The SRVF for a continuous 
%   curve b(t) is defined by
%         q(t) = b'(t) / sqrt(|b'(t)|)
%   The input SRVF Q for this function should be a 2xN array. The return
%   value CURVE is an array of the same size.

[dim,N] = size(q);

if isempty(varargin)
    t = 0;
else 
    t = varargin{1};
    if length(t) ~= size(q,2)
        error('The length of t has to be the same as the number of columns in q!');
    end
end

qnorm = sqrt( sum(q.^2) );

p = zeros(dim,N);
if isscalar(t) % Uniformly discretized with const spacing h.
    h = 1/(N-1);
    for i = 1:dim
        p(i,:) = h * cumtrapz( q(i,:).*qnorm );
    end
else % Nonuniformly discretized at grid points t.
    for i = 1:dim
        p(i,:) = cumtrapz( t, q(i,:).*qnorm );
    end
end
