function q = curve_to_q(p,varargin)
%CURVE_TO_SRVF computes the Square-Root Velocity Function for the curve.
%
%   Q = CURVE_TO_SRVF(CURVE) computes the Square-Root Velocity Function
%   (SRVF) representation of a given curve (see Srivastava et al, PAMI 2011)
%   using centered finite difference approximation for the discrete
%   derivatives. The SRVF for a continuous curve b(t) is defined by
%         q(t) = b'(t) / sqrt(|b'(t)|)
%   The input CURVE for this function should be a dxN array, and the return
%   value Q is an array of the same size.
%
%   Optional parameters are T, the parameterization array [0 T(2) ... T(N-1) 1]
%   of size 1xN; CURVE_DERIV, the derivate of the CURVE at parameterization
%   points T, dxN array; GAMMA_DERIV, the derivative of a reparameterization
%   function gamma at points T, 1xN array.
%
%   Q = CURVE_TO_SRVF(CURVE, T, CURVE_DERIV, GAMMA_DERIV)
%

[dim,N] = size(p);

t = 0;  dp = [];  gamma_dot = [];
if nargin > 1
    t = varargin{1};
    if length(t) ~= N
        error('The length of T has to be the same as the number of columns/nodes in CURVE!');
    end
    if nargin > 2
        dp = varargin{2};
        if (size(dp,1) ~= dim) && (size(dp,2) ~= N) && ~isempty(dp)
            error('CURVE_DERIV array has to have the same size as CURVE!')
        end
        if nargin > 3
            gamma_dot = varargin{3};
            if (length(gamma_dot) ~= N) && ~isempty(gamma_dot)
                error('The length of GAMMA_DERIV has to be the same as the number of columns/nodes in CURVE!')
            end
            if any( gamma_dot <= 0 )
                error('All elements of GAMMA_DERIV have to be positive!')
            end
        end
    end
end


if isempty(dp)
    dp = num_deriv(p,t); % Numerical derivative of p.
    
    % if p is closed curve, first and last nodes are equal: use centered diff.
    if norm(p(:,1)-p(:,N)) < 100*eps
        if isscalar(t) % The uniform case.
            h = 1 / (N-1);
            dp(:,1) = (p(:,2) - p(:,N-1)) / (2*h);
        else % The nonuniform case.
            h = [ t(N)-t(N-1) t(2)-t(1) ];
            three_nodes = [p(:,N-1) p(:,1) p(:,2)];  % Note: p(:,1) == p(:,N)
            dp(:,1) = centered_finite_difference(three_nodes,h);
        end
        dp(:,N) = dp(:,1);
    end
end

norm_dp = sqrt( sum(dp.^2,1) );

if dim == 1
    q = sign(dp) .* sqrt(norm_dp);
else % dim = 2,3,...
    q = bsxfun( @rdivide, dp, sqrt(norm_dp) );
end

if ~isempty( gamma_dot )
    q = bsxfun( @times, q, sqrt(gamma_dot) );
end

% Normalize the SRVF function Q so that L2_norm(q,t) == 1.
% This normalization is necessitated by the numerical discrepancy.
% The unit-norm condition should be satisfied w/o normalization in theory.
if isscalar(t)
    norm_q = L2_inner_prod(q,q)^0.5;
else
    norm_q = L2_inner_prod(q,q,t)^0.5;
end
q = q / norm_q;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function df = num_deriv(f,t)

[dim,N] = size(f);
if isscalar(t) % The uniform case.
    h = 1 / (N-1);
    df = gradient(f,h);
else % Nonuniform case: grid pts t provided, lead to nonuniform spacing h.
    df = zeros(dim,N);
    h = t(2:N) - t(1:N-1);
    df(:,2:N-1) = centered_finite_difference(f,h);
    df(:,1) = (f(:,2) - f(:,1)) / h(1);
    df(:,N) = (f(:,N) - f(:,N-1)) / h(N-1);
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function df = centered_finite_difference(f,h)

N = size(f,2);
w1 = h(1:N-2) ./ h(2:N-1);   
w2 = 1./w1;   
avg_h = h(1:N-2)+h(2:N-1);

df = bsxfun( @times,  w1./avg_h,  f(:,3:N)  - f(:,2:N-1) ) + ...
     bsxfun( @times,  w2./avg_h, f(:,2:N-1) - f(:,1:N-2) );

end
